@@ -1,50 +1,25 @@
 from collections import defaultdict
-import inspect
 from functools import wraps
+from typing import Dict
 
+from .utils import get_func_signatures
 from .typesystem import TypeSystem
 
 
 class DispatchError(Exception):
     pass
 
+
 class MultiDispatch:
-    """Create a decorator attached to a dispatch group,
-    that when applied to a function, enables multiple-dispatch for it.
-
-    Example:
-        >>> from runtype import Dispatch
-        >>> dp = Dispatch()
-
-        >>> @dp
-        ... def add1(i: Optional[int]):
-        ...     return i + 1
-
-        >>> @dp
-        ... def add1(s: Optional[str]):
-        ...     return s + "1"
-
-        >>> @dp
-        ... def add1(a):  # Any, which is the least-specific
-        ...     return (a, 1)
-
-        >>> add1(1)
-        2
-
-        >>> add1("1")
-        11
-
-        >>> add1(1.0)
-        (1.0, 1)
-
+    """Creates a dispatch group for multiple dispatch
 
+    Parameters:
+        typesystem - instance for interfacing with the typesystem
+        test_subtypes: indices of params that should be matched by subclass instead of isinstance.
     """
+
     def __init__(self, typesystem: TypeSystem, test_subtypes: set = set()):
-        """Parameters:
-            typesystem - instance for interfacing with the typesystem
-            test_subtypes: indices of params that should be matched by subclass instead of isinstance.
-        """
-        self.roots = defaultdict(TypeTree)
+        self.roots : Dict[str, TypeTree] = defaultdict(TypeTree)
         self.typesystem = typesystem
         self.test_subtypes = test_subtypes
 
@@ -110,7 +85,7 @@ class TypeTree:
 
 
     def define_function(self, f):
-        for signature in self.get_func_signatures(f):
+        for signature in get_func_signatures(self.typesystem, f):
             node = self.root
             for t in signature:
                 node = node.follow_type[t]
@@ -121,30 +96,6 @@ class TypeTree:
             node.func = f, signature
 
 
-    def get_func_signatures(self, f):
-        sig = inspect.signature(f)
-        typesigs = []
-        typesig = []
-        for p in sig.parameters.values():
-            # if p.kind is p.VAR_KEYWORD or p.kind is p.VAR_POSITIONAL:
-            #     raise TypeError("Dispatch doesn't support *args or **kwargs yet")
-
-            t = p.annotation
-            if t is sig.empty:
-                t = self.typesystem.default_type
-            else:
-                # Canonize to detect more collisions on construction, instead of during dispatch
-                t = self.typesystem.canonize_type(t)
-
-            if p.default is not p.empty:
-                # From now on, everything is optional
-                typesigs.append(list(typesig))
-
-            typesig.append(t)
-
-        typesigs.append(typesig)
-        return typesigs
-
     def choose_most_specific_function(self, *funcs):
         issubclass = self.typesystem.issubclass
 
